from colorcodetools.cmyk.validate import is_cmyk
from colorcodetools.rgb.convert import RgbToHex
from typing import Union


class CmykToRgb:
    @staticmethod
    def cmyk_to_rgb(cmyk: str, prefix: bool = True) -> Union[bool, str]:
        check_cmyk = is_cmyk(cmyk)
        if not check_cmyk:
            return False
        cmyk_list: list = cmyk.split("/")
        if prefix:
            inverted_cmyk: str = 'rgb('
        else:
            inverted_cmyk: str = '('
        try:
            c = int(cmyk_list[0]) / 100
            m = int(cmyk_list[1]) / 100
            y = int(cmyk_list[2]) / 100
            k = int(cmyk_list[3]) / 100
        except:
            return False
        r = round(255 * (1 - c) * (1 - k))
        g = round(255 * (1 - m) * (1 - k))
        b = round(255 * (1 - y) * (1 - k))
        inverted_cmyk += f'{r},{g},{b})'
        return inverted_cmyk


class CmykToHex:
    @staticmethod
    def cmyk_to_hex(cmyk: str, prefix: bool = True) -> Union[bool, str]:
        cmyk_to_rgb: CmykToRgb = CmykToRgb()
        rgb_to_hex: RgbToHex = RgbToHex()
        inverted_cmyk = rgb_to_hex.rgb_to_hex(cmyk_to_rgb.cmyk_to_rgb(cmyk, prefix=False), prefix=prefix)
        return inverted_cmyk



from string import punctuation


def is_hex(hexcode: str) -> bool:
    if hexcode.startswith('#'):
        hex_check: str = hexcode.replace('#', '')
    else:
        hex_check: str = hexcode
    if len(hex_check) != 8 and len(hex_check) != 6 and len(hex_check) != 3:
        return False
    for e in list(hex_check):
        if e >= "g" or e in punctuation:
            return False
    return True


def is_in_vga(hexcode: str) -> bool:
    if hexcode.startswith('#'):
        hex_check: str = hexcode.replace('#', '')
    else:
        hex_check: str = hexcode
    if len(hex_check) != 6 and len(hex_check) != 3:
        return False
    if hex_check.lower() in __config_is_vga():
        return True
    return False


def is_in_netscape(hexcode: str) -> bool:
    if hexcode.startswith('#'):
        hex_check: str = hexcode.replace('#', '')
    else:
        hex_check: str = hexcode
    if not is_hex(hex_check):
        return False
    hex_list: list = list(hex_check.upper())
    if len(hex_list) == 3:
        for e in hex_list:
            if e not in ['0', '3', '6', '9', 'C', 'F']:
                return False
    else:
        for e in range(0, len(hex_list), 2):
            if str(hex_list[e] + hex_list[e + 1]).upper() not in ['00', '33', '66', '99', 'CC', 'FF']:
                return False
    return True

def __config_is_vga() -> list:
    return ['000000', '800000', '008000', '808000', '000080', '800080', '008080', 'c0c0c0', '808080', 'ff0000',
            '00ff00', 'ffff00', '0000ff', 'ff00ff', '00ffff', 'ffffff']



def is_cmyk(cmyk: str) -> bool:
    try:
        cmyk_split = cmyk.split("/")
    except:
        return False
    for e in cmyk_split:
        try:
            int(e)
        except:
            return False
        if len(e) > 3:
            return False
        if int(e) > 100 or int(e) < 0:
            return False
    return True


def is_single_cmyk(cmyk: int) -> bool:
    if 100 >= cmyk >= 0:
        return True
    else:
        return False



from colorcodetools.cmyk.validate import is_cmyk, is_single_cmyk
from typing import Union
from colorcodetools.cmyk.config import config


def invert_cmyk(cmyk: str) -> Union[bool, str]:
    check_cmyk: bool = is_cmyk(cmyk=cmyk)
    if not check_cmyk:
        return False
    config_data = config()
    inverted_cmyk: str = ''
    cmyk = cmyk.split("/")
    count: int = 0
    for e in list(cmyk):
        if count != len(list(cmyk)) - 1:
            inverted_cmyk += config_data[e.lower()] + "/"
            count += 1
        else:
            inverted_cmyk += config_data[e.lower()]
    return inverted_cmyk


def invert_single_cmyk(cmyk: int) -> Union[bool, int]:
    check_cmyk: bool = is_single_cmyk(cmyk)
    if not check_cmyk:
        return False
    config_data = config()
    inverted_cmyk: int = config_data[str(cmyk).strip()]
    return inverted_cmyk



def is_rgb(rgb: str, a: float = 1.0) -> bool:
    try:
        rgb_split = rgb.replace("(", "")
        rgb_split = rgb_split.replace(")", "")
        rgb_split = rgb_split.split(",")
    except:
        return False
    if len(rgb_split) != 3:
        return False
    for e in rgb_split:
        try:
            if int(e) > 255 or int(e) < 0:
                return False
        except:
            return False
    if a > 1.0 or a < 0.0:
        return False
    return True


def is_single_rgb(rgb: int) -> bool:
    if 255 >= rgb >= 0:
        return True
    else:
        return False



from colorcodetools.rgb.validate import is_rgb, is_single_rgb
from typing import Union
from colorcodetools.rgb.config import config


def invert_rgb(rgb: str, a: float = 1.0, prefix: bool = True) -> Union[bool, str]:
    check_rgb: bool = is_rgb(rgb, a)
    if not check_rgb:
        return False
    config_data = config()
    rgb: str = rgb.replace("(", "")
    rgb = rgb.replace(")", "")
    if prefix:
        if a == 1.0:
            inverted_rgb: str = 'rgb('
        else:
            inverted_rgb: str = 'rgba('
    else:
        inverted_rgb: str = '('
    if a == 1.0:
        count: int = 0
        for e in rgb.split(","):
            count += 1
            if count == 3:
                inverted_rgb += config_data[e.strip()]
            else:
                inverted_rgb += config_data[e.strip()] + ", "
        inverted_rgb += ")"
    else:
        for e in rgb.split(","):
            inverted_rgb += config_data[e.strip()] + ", "
        inverted_rgb += str(a) + ")"
    return inverted_rgb


def invert_single_rgb(rgb: int) -> Union[bool, int]:
    check_rgb: bool = is_single_rgb(rgb)
    if not check_rgb:
        return False
    config_data = config()
    inverted_rgb: int = config_data[str(rgb).strip()]
    return inverted_rgb



from colorcodetools.hex.validate import is_hex
from typing import Union


class HexToRgb:
    @staticmethod
    def hex_to_rgb(hexcode: str, prefix: bool = True) -> Union[bool, str]:
        check_hex: bool = is_hex(hexcode)
        if not check_hex:
            return False
        if hexcode.startswith('#'):
            hexcode = hexcode.replace('#', '')
        else:
            hexcode = hex
        if prefix and len(hexcode) == 8:
            converted_hex: str = 'rgba('
        elif prefix:
            converted_hex: str = 'rgb('
        else:
            converted_hex: str = '('
        hex_values: list = list(hexcode)
        count: int = 0
        index: int = 0
        if len(hexcode) == 3:
            while index < 3:
                if count == 2:
                    converted_hex += str(int(hex_values[index] + hex_values[index], 16))
                else:
                    converted_hex += str(int(hex_values[index] + hex_values[index], 16)) + ", "
                index += 1
                count += 1
        elif len(hexcode) == 6:
            while index < 6:
                if count == 4:
                    converted_hex += str(int(hex_values[index] + hex_values[index + 1], 16))
                else:
                    converted_hex += str(int(hex_values[index] + hex_values[index + 1], 16)) + ", "
                index += 2
                count += 2
        else:
            while index < 8:
                if count == 6:
                    if hex_values[index].lower() == "f" and hex_values[index + 1].lower() == "f":
                        converted_hex += "1.0"
                    else:
                        converted_hex += str(round(int(hex_values[index] + hex_values[index + 1], 16) / 16 / 16, 3))
                else:
                    converted_hex += str(int(hex_values[index] + hex_values[index + 1], 16)) + ", "
                index += 2
                count += 2
        converted_hex += ")"
        return converted_hex


class HexToCmyk:
    @staticmethod
    def hex_to_cmyk(hexcode: str) -> Union[bool, str]:
        check_hex = is_hex(hexcode)
        if not check_hex:
            return False
        if hexcode.startswith('#'):
            hexcode = hexcode.replace('#', '')
        if len(hexcode) > 6:
            return False
        hex_list: list = list(hexcode)
        if len(hexcode) == 3:
            c: Union[int, float] = int(hex_list[0] + hex_list[0], 16)
            m: Union[int, float] = int(hex_list[1] + hex_list[1], 16)
            y: Union[int, float] = int(hex_list[2] + hex_list[2], 16)
        else:
            c: Union[int, float] = int(hex_list[0] + hex_list[1], 16)
            m: Union[int, float] = int(hex_list[2] + hex_list[3], 16)
            y: Union[int, float] = int(hex_list[4] + hex_list[5], 16)
        c /= 255
        m /= 255
        y /= 255
        k = 1 - max(c, m, y)
        c = (1 - c - k) / (1 - k)
        m = (1 - m - k) / (1 - k)
        y = (1 - y - k) / (1 - k)
        inverted_hex: str = f'{round(c * 100)}, {round(m * 100)}, {round(y * 100)}, {round(k * 100)}'
        return inverted_hex



from colorcodetools.rgb.validate import is_rgb
from typing import Union


class RgbToHex:
    def rgb_to_hex(self, rgb: str, a: float = 1.0, prefix: bool = True) -> Union[bool, str]:
        check_rgb: bool = is_rgb(rgb, a)
        if not check_rgb:
            return False
        rgb: str = rgb.replace("(", "")
        rgb = rgb.replace(")", "")
        if prefix:
            converted_rgb: str = '#'
        else:
            converted_rgb: str = ''
        rgb_values: list = rgb.split(",")
        for e in rgb_values:
            converted_rgb += self.__number_to_hex_part(int(e))
        if 1.0 > a >= 0.0:
            a = round(a * 100)
            converted_rgb += self.__number_to_hex_part(a)
            return converted_rgb
        elif a == 1.0:
            return converted_rgb
        else:
            return False

    def __number_to_hex_part(self, number: int):
        result = str(number / 16).split(".")
        return self.__first_to_hex_char(int(result[0])) + self.__second_to_hex_char(int(result[1]))

    @staticmethod
    def __first_to_hex_char(number: int) -> str:
        if number == 10:
            return "A"
        elif number == 11:
            return "B"
        elif number == 12:
            return "C"
        elif number == 13:
            return "D"
        elif number == 14:
            return "E"
        elif number == 15:
            return "F"
        else:
            return str(number)

    @staticmethod
    def __second_to_hex_char(number: int) -> str:
        number = float("0." + str(number))
        result = round(number * 16)
        if result == 10:
            return "A"
        elif result == 11:
            return "B"
        elif result == 12:
            return "C"
        elif result == 13:
            return "D"
        elif result == 14:
            return "E"
        elif result == 15:
            return "F"
        else:
            return str(result)


class RgbToCymk:
    @staticmethod
    def rgb_to_cymk(rgb: str) -> Union[bool, str]:
        rgb_scale = 255
        cymk_scale = 100
        check_rgb: bool = is_rgb(rgb)
        if not check_rgb:
            return False
        rgb: str = rgb.replace("(", "")
        rgb = rgb.replace(")", "")
        rgb_values: list = rgb.split(",")
        r = int(rgb_values[0])
        g = int(rgb_values[1])
        b = int(rgb_values[2])
        if (r, g, b) == (0, 0, 0):
            return f"0, 0, 0, {cymk_scale}"
        c = 1 - r / rgb_scale
        m = 1 - g / rgb_scale
        y = 1 - b / rgb_scale

        min_cmy = min(c, m, y)
        c = (c - min_cmy) / (1 - min_cmy)
        m = (m - min_cmy) / (1 - min_cmy)
        y = (y - min_cmy) / (1 - min_cmy)
        k = min_cmy
        inverted_rgb: str = f"{int(c * cymk_scale)}, {int(m * cymk_scale)}, {int(y * cymk_scale)}, " \
                            f"{int(k * cymk_scale)}"
        return inverted_rgb



from colorcodetools.hex.validate import is_hex
from typing import Union
from colorcodetools.hex.config import config


def invert_hex(hexcode: str, prefix: bool = True) -> Union[bool, str]:
    check_hex: bool = is_hex(hexcode=hexcode)
    if not check_hex:
        return False
    config_data = config()
    hexcode: str = hexcode.replace('#', '')
    if prefix:
        inverted_hex: str = '#'
    else:
        inverted_hex: str = ''
    count: int = 0
    for e in list(hexcode):
        if count < 6:
            inverted_hex += config_data[e.lower()]
            count += 1
        else:
            inverted_hex += e
    return inverted_hex



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='colorcodetools',
    version='0.3.0',
    author='Keine Ahnung',
    author_email='kontakt@keineahnung.eu',
    description='Some powerful color tools',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/TheKeineAhnung/colorcodes',
    project_urls={
        'Bug Tracker': 'https://github.com/TheKeineAhnung/colorcodes/issues',
    },
    license='LICENSE',
    classifiers=(
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    packages=setuptools.find_packages(),
    python_requires='>=3.6'
)